<?php
/**
 * Plugin Name: WooCommerce Payrexx Gateway
 * Description: Accept many different payment methods on your store using Payrexx
 * Author: Payrexx
 * Author URI: https://payrexx.com
 * Version: 1.7.5
 * Requires at least: 4.4
 * Tested up to: 6.0
 * WC requires at least: 3.8.1
 * WC tested up to: 5.3
 */


add_action('plugins_loaded', 'woocommerce_gateway_payrexx_init' );

function woocommerce_gateway_payrexx_init() {
    if (!class_exists( 'WooCommerce' ) ) {
        return;
    }

    wc_offline_gateway_init();
}

function wc_offline_gateway_init()
{
    class WC_Payrexx_Gateway extends WC_Payment_Gateway
    {
        public $enabled;
        public $title;
        public $description;
        public $instance;
        public $platform;
        public $apiKey;
        public $prefix;
        public $logos;
        public $subscriptionLogos;
        public $lang;

        public function __construct()
        {
            load_plugin_textdomain('wc-payrexx-gateway', false, basename(dirname(__FILE__)) . '/languages');
            $this->lang = ['en', 'de', 'it', 'fr', 'nl', 'pt', 'tr'];
            $this->id = 'payrexx';
            $this->method_title = __('Payrexx', 'woocommerce');
            $this->method_description = __('Accept many different payment methods on your store using Payrexx.', 'woocommerce');

            // Add subscription support if activated
            if ($this->get_option('subscriptions_enabled') === 'yes') {
                $this->supports = array_merge($this->supports, array(
                    'subscriptions',
                    'subscription_cancellation',
                    'subscription_suspension',
                    'subscription_reactivation',
                    'subscription_amount_changes',
                    'subscription_date_changes',
                    'subscription_payment_method_change',
                    'subscription_payment_method_change_customer',
                    'subscription_payment_method_change_admin',
                    'multiple_subscriptions',
                ));
                // Filter to extend description and title in case of recurring payment
                add_filter('woocommerce_gateway_title', array($this, 'mutateTitle'), 10, 2);
                add_filter('woocommerce_gateway_description', array($this, 'mutateDescription'), 10, 2);
            }

            $this->init_form_fields();
            $this->init_settings();

            $this->enabled = $this->get_option('enabled');
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->instance = $this->get_option('instance');
            $this->platform = $this->get_option('platform');
            $this->apiKey = !empty($this->get_option('apiKey')) ? $this->get_option('apiKey') : $this->get_option('sid');
            $this->prefix = $this->get_option('prefix');
            $this->logos = $this->get_option('logos');
            $this->subscriptionLogos = $this->get_option('subscription_logos');
            $this->lookAndFeelId = $this->get_option('lookAndFeelId');


            if (isset($_GET['payrexx_error'])) {

                // Delete old Gateway using order metadata

                if (isset($_GET['order_id'])) {
                    $gatewayId = intval(get_post_meta($_GET['order_id'], 'payrexx_gateway_id', true));
                    $this->deleteGatewayById($gatewayId);
                }
                wc_print_notice(__('Payment failed. Please choose another method.', 'wc-payrexx-gateway'), 'error');
            }

            $this->method_title = __('Payrexx', 'woocommerce');
            $this->method_description = __('Accept many different payment methods on your store using Payrexx.', 'woocommerce');

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_api_wc_payrexx_gateway', array($this, 'check_webhook_response'));
            add_action('woocommerce_scheduled_subscription_payment_' . $this->id, array($this, 'process_recurring_payment'), 10, 2);
        }

        public function get_icon()
        {
            $style = version_compare(WC()->version, '2.6', '>=') ? 'margin-left: 0.3em;' : '';

            $subscriptionPayment = $this->isSubscriptionPayment();
            $paymentMethodChange = $this->isPaymentMethodChange();

            $hideStyle = $subscriptionPayment ? 'display: none' : '';
            $logos = $paymentMethodChange ? $this->subscriptionLogos : $this->logos;

            $icon = '';
            foreach ($logos as $logo) {
                if ($subscriptionPayment && !$paymentMethodChange && in_array($logo, $this->subscriptionLogos)) {
                    $icon .= '<img class="tokenization" src="' . WC_HTTPS::force_https_url(plugins_url('cardicons/card_' . $logo . '.svg', __FILE__)) . '" alt="' . $logo . '" id="' . $logo . '" width="32" style="'.$style.'" />';
                }
                $icon .= '<img class="onetime" src="' . WC_HTTPS::force_https_url(plugins_url('cardicons/card_' . $logo . '.svg', __FILE__)) . '" alt="' . $logo . '" id="' . $logo . '" width="32" style="' . $style . $hideStyle . '" />';
            }

            // Add a wrapper around the images to allow styling
            return apply_filters('woocommerce_gateway_icon', '<span class="icon-wrapper">' . $icon . '</span>', $this->id);
        }

        /**
         * Initialize Gateway Settings Form Fields
         */
        public function init_form_fields()
        {
            $this->form_fields = include('includes/settings-payrexx.php');
        }

        public function process_payment($order_id)
        {
            $order = wc_get_order($order_id);
            // Return thankyou redirect
            return array(
                'result' => 'success',
                'redirect' => $this->get_payrexx_gateway($order)
            );
        }

        public function payment_scripts()
        {
            if (!is_cart() && !is_checkout() && !isset($_GET['pay_for_order']) && !is_add_payment_method_page()) {
                return;
            }
        }

        public function check_webhook_response()
        {
            try {
                $resp = $_REQUEST;
                $orderId = $resp['transaction']['invoice']['referenceId'];

                if (!empty($this->prefix) && strpos($orderId, $this->prefix) === false) {
                    return;
                }

                $arr = explode('_', $orderId);
                $orderId = end($arr);

                if (!isset($resp['transaction']['status'])) {
                    throw new \Exception('Missing transaction status');
                }

                $status = $this->get_payrexx_status($resp['transaction']['id']);
                if ($status !== $resp['transaction']['status']) {
                    throw new \Exception('Fraudulent transaction status');
                }

                // Check if subscription to handle accordingly
                $parentSubscription = null;
                if (!empty($resp['transaction']['preAuthorizationId'])) {
                    $subscriptions = wcs_get_subscriptions_for_order($orderId, array( 'order_type' => 'any' ));
                    foreach ($subscriptions as $subscription) {
                        $parentSubscription = $subscription;
                    }

                    // OrderId is subscription id in case of payment method change
                    if (!$parentSubscription) {
                        $parentSubscription = new WC_Subscription($orderId);
                    }

                    $orderId = $parentSubscription->get_last_order();
                }

                $order = new WC_Order($orderId);

                if (!$order) {
                    throw new \Exception('Fraudulent request');
                }

                $this->register_autoloader();

                switch ($status) {
                    case \Payrexx\Models\Response\Transaction::WAITING:
                        $newTransactionsStatus = 'on-hold';
                        $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);

                        if ($order->get_status() === $newTransactionsStatus) {
                            break;
                        }

                        $order->update_status($newTransactionsStatus, __('Awaiting payment', 'wc-payrexx-gateway'));
                        break;
                    case \Payrexx\Models\Response\Transaction::CONFIRMED:
                        if ($order->has_status(['processing', 'completed'])) {
                            break;
                        }

                        $order->update_meta_data('payrexx_payment_method', $resp['transaction']['payment']['brand']);
                        $order->save();

                        $order->payment_complete();
                        // Remove cart
                        WC()->cart->empty_cart();
                        break;
                    case \Payrexx\Models\Response\Transaction::AUTHORIZED:
                        $parentSubscription->update_meta_data('payrexx_auth_transaction_id', $resp['transaction']['preAuthorizationId']);
                        $parentSubscription->save();
                        break;
                    case \Payrexx\Models\Response\Transaction::REFUNDED:
                        $newTransactionsStatus = 'refunded';
                        $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);
                        if ($order->get_status() === $newTransactionsStatus) {
                            break;
                        }
                        $order->update_status($newTransactionsStatus, __('Payment was fully refunded', 'wc-payrexx-gateway'));
                        break;
//                    case \Payrexx\Models\Response\Transaction::PARTIALLY_REFUNDED:
//                        if ($order->get_status() === 'refunded') {
//                            break;
//                        }
//                        $order->update_status('refunded', __('Payment was partially refunded', 'wc-payrexx-gateway'));
//                        break;
                    case \Payrexx\Models\Response\Transaction::CANCELLED:
                    case \Payrexx\Models\Response\Transaction::EXPIRED:
                    case \Payrexx\Models\Response\Transaction::DECLINED:
                        $newTransactionsStatus = 'cancelled';
                        $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);

                        if ($order->has_status(['processing', 'completed', $newTransactionsStatus])) {
                            break;
                        }

                        $order->update_status($newTransactionsStatus, __('Payment was cancelled by the customer', 'wc-payrexx-gateway'));
                        break;
                    case \Payrexx\Models\Response\Transaction::ERROR:
                        $newTransactionsStatus = 'failed';
                        $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);
                        if ($order->has_status(['processing', 'completed', $newTransactionsStatus])) {
                            break;
                        }
                        $order->update_status($newTransactionsStatus, __('An error occured while processing this payment', 'wc-payrexx-gateway'));
                        break;
                }
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
            }
        }

        public function get_payrexx_gateway($order_id)
        {
            $productPriceIncludesTax = ('yes' === get_option( 'woocommerce_prices_include_tax'));

            $order = new WC_Order($order_id);
            $totalAmount = floatval($order->get_total());
            $currency = get_woocommerce_currency();

            $basketAmount = 0;

            $cartItems = WC()->cart->get_cart();
            $products = [];
            foreach ($cartItems as $item) {
                $amount = $item['data']->get_price();
                $basketAmount += $amount * $item['quantity'];
                $products[] = [
                    'name' => $item['data']->get_title(),
                    'description' => $item['data']->get_short_description(),
                    'quantity' => $item['quantity'],
                    'amount' => $amount * 100,
                    'sku' => $item['data']->get_sku(),
                ];
            }

            $shipping = WC()->cart->get_shipping_total();
            $shippingTax = WC()->cart->get_shipping_tax();
            if ($shipping || $shippingTax) {
                $amount = $shipping + $shippingTax;
                $basketAmount += $amount;
                $products[] = [
                    'name' => 'Shipping',
                    'quantity' => 1,
                    'amount' => $amount * 100,
                ];
            }

            $discount = WC()->cart->get_discount_total();
            $discountTax = WC()->cart->get_discount_tax();
            if ($discount) {
                $amount = $discount;
                $amount += !$productPriceIncludesTax ? 0 : $discountTax;
                $basketAmount += $amount;
                $products[] = [
                    'name' => 'Discount',
                    'quantity' => 1,
                    'amount' => $amount * -100,
                ];
            }

            $fee = WC()->cart->get_fee_total();
            $feeTax = WC()->cart->get_fee_tax();
            if ($fee) {
                $amount = $fee;
                $amount += !$productPriceIncludesTax ? 0 : $feeTax;
                $basketAmount += $amount;
                $products[] = [
                    'name' => 'Fee',
                    'quantity' => 1,
                    'amount' => $amount * 100,
                ];
            }

            $taxAmount = WC()->cart->get_cart_contents_tax();
            if ($taxAmount && !$productPriceIncludesTax) {
                $amount = $taxAmount;
                $basketAmount += $amount;
                $products[] = [
                    'name' => 'Tax',
                    'quantity' => 1,
                    'amount' => $amount * 100,
                ];
            }

            $first_name = $order->get_billing_first_name();
            $last_name = $order->get_billing_last_name();
            $company = $order->get_billing_company();
            $address_1 = $order->get_billing_address_1();
            $postcode = $order->get_billing_postcode();
            $city = $order->get_billing_city();
            $country = $order->get_billing_country();
            $phone = $order->get_billing_phone();
            $email = $order->get_billing_email();

            $payrexx = $this->getPayrexxInterface();
            $gateway = new \Payrexx\Models\Request\Gateway();

            if ($currency == "") {
                $currency = "USD";
            }
            $gateway->setCurrency($currency);
            $gateway->setValidity(15);

            $gateway->setSuccessRedirectUrl($this->get_return_url($order));
            $gateway->setFailedRedirectUrl(wc_get_cart_url() . '?payrexx_error=1&order_id=' . $order->get_id());
            $gateway->setCancelRedirectUrl(wc_get_cart_url() . '?payrexx_error=1&order_id=' . $order->get_id());


            $gateway->setPsp([]);
            $gateway->setSkipResultPage(true);

            // Maybe register for tokenization
            $subscriptionPayment = $this->isSubscriptionPayment();
            $paymentMethodChange = $this->isPaymentMethodChange();
            if (($paymentMethodChange || $subscriptionPayment) && !empty($_POST['payrexx-allow-recurring'])) {
                $gateway->setPreAuthorization(true);

                // No immmediate charge should happen for payment method changes
                // Also not if totalAmount is 0 and must be artificially elevated for tokenizations to work
                if (!$paymentMethodChange && $totalAmount) {
                    $gateway->setChargeOnAuthorization(true);
                }

                // The amount is artificially elevated because the Gateway creation always needs an amount
                if ($paymentMethodChange || !$totalAmount) {
                    $totalAmount = 0.50;
                }

            } elseif ($paymentMethodChange || $subscriptionPayment) {
                $subscriptions = wcs_get_subscriptions_for_order($order_id, array( 'order_type' => 'parent' ));
                foreach ($subscriptions as $subscription) {
                    $subscription->set_requires_manual_renewal(true);
                    $subscription->save();
                }

                // In case of manual subscription and no amount, the order is completed
                if (!$totalAmount) {
                    return $this->get_return_url($order);
                }
            }

            if ($totalAmount && $totalAmount === floatval($basketAmount) && false) {
                $gateway->setBasket($products);
            } else {
                // Basket backup transmition option
                $desc = [];
                foreach ($products as $product) {
                    $desc[] = implode(' ', [
                        $product['name'],
                        $product['quantity'],
                        'x',
                        number_format($product['amount'] / 100, 2, '.'),
                    ]);
                }
                $gateway->setPurpose(implode('; ', $desc));
            }

            $gateway->setAmount($totalAmount * 100);

            if ($this->prefix == "") {
                $gateway->setReferenceId($order->get_id());
            } else {
                $gateway->setReferenceId($this->prefix . '_' . $order->get_id());
            }

            if ($this->lookAndFeelId != "") {
                $gateway->setLookAndFeelProfile($this->lookAndFeelId);
            }

            $gateway->addField($type = 'title', $value = '');
            $gateway->addField($type = 'forename', $value = $first_name);
            $gateway->addField($type = 'surname', $value = $last_name);
            $gateway->addField($type = 'company', $value = $company);
            $gateway->addField($type = 'street', $value = $address_1);
            $gateway->addField($type = 'postcode', $value = $postcode);
            $gateway->addField($type = 'place', $value = $city);
            $gateway->addField($type = 'country', $value = $country);
            $gateway->addField($type = 'phone', $value = $phone);
            $gateway->addField($type = 'email', $value = $email);
            $gateway->addField($type = 'custom_field_1', $value = $order->get_id(), $name = 'WooCommerce ID');
            $gateway->addField($type = 'custom_field_2', $value = get_locale(), $name = 'WooCommerce ID');

            try {
                $response = $payrexx->create($gateway);
                $order->update_meta_data('payrexx_gateway_id', $response->getId());
                $order->save();

                $language = substr(get_locale(), 0, 2);
                !in_array($language, $this->lang) ? $language = $this->lang[0] : null;
                $res = str_replace('?', $language . '/?', $response->getLink());
                return $res;
            } catch (\Payrexx\PayrexxException $e) {
                print $e->getMessage();
            }
        }

        public function get_payrexx_status($transactionId)
        {
            if (!$transactionId) {
                return false;
            }

            $payrexx = $this->getPayrexxInterface();

            $transaction = new \Payrexx\Models\Request\Transaction();
            $transaction->setId($transactionId);
            try {
                $response = $payrexx->getOne($transaction);
                return $response->getStatus();
            } catch (\Payrexx\PayrexxException $e) {
                print $e->getMessage();
            }
        }

        /**
         * @param float $amount
         * @param \WC_Order $renewal
         */
        public function process_recurring_payment($amount, $renewal)
        {
            $subscriptions = wcs_get_subscriptions_for_order($renewal, array('order_type' => 'any'));

            foreach ($subscriptions as $subscription) {

                // Get tokenization from the subscription (new way)
                $tokenizationId = intval(get_post_meta($subscription->get_id(), 'payrexx_auth_transaction_id', true));

                if (!$tokenizationId) {
                    // Get tokenization id from the last order except this renewal (old way)
                    $related_orders = $subscription->get_related_orders('ids');

                    foreach ($related_orders as $order_id) {
                        if ($order_id === $renewal->get_id()) continue;
                        $last_order_id = $order_id;
                        break;
                    }
                    $tokenizationId = intval(get_post_meta($last_order_id, 'payrexx_auth_transaction_id', true));

                    $subscription->update_meta_data('payrexx_auth_transaction_id', $tokenizationId);
                    $subscription->save();
                }

                // Both must be given to do a valid recurring transaction
                $payrexx = $this->getPayrexxInterface();
                $transaction = new \Payrexx\Models\Request\Transaction();
                $transaction->setId($tokenizationId);
                $transaction->setAmount(floatval($amount) * 100);
                try {
                    $payrexx->charge($transaction);
                    continue;
                } catch (\Payrexx\PayrexxException $e) {
                }

                // Recurring payment failed if we reach this point
                $subscription->payment_failed();
            }
        }

        public function mutateTitle($title, $id) {
            // Ignore if this is not a Payrexx based transaction/subscription
            if ($id !== 'payrexx') {
                return $title;
            }

            if (!$this->isPaymentMethodChange()) return $title;
            return $this->get_option('subscriptions_title');
        }

        /**
         * @param $description
         * @param $id
         * @return mixed
         */
        public function mutateDescription($description, $id)
        {
            // Ignore if this is not a Payrexx based transaction/subscription
            if ($id !== 'payrexx') {
                return $description;
            }

            $html = '';
            if (strlen($description) > 0) {
                $html .= wpautop($description);
            }

            // Handle change payment method action
            if ($this->isPaymentMethodChange()) {
                $html .= '<input type="hidden" name="payrexx-allow-recurring" id="payrexx-allow-recurring" value="1" />';
                return $html;
            }

            if (!$this->isSubscriptionPayment()) return false;

            // do it the messy way as WCS is using wpautop on this string
            $html .= '<label for="payrexx-allow-recurring">';
            $html .= '<input type="checkbox" checked name="payrexx-allow-recurring" id="payrexx-allow-recurring" value="1" />';
            $html .= $this->get_option('subscriptions_user_desc') . '</label>';
            // Add logic to show cards depending on checkbox
            $html .= '<script type="text/javascript">
                  jQuery(function() {
                    jQuery("#payrexx-allow-recurring").on("change", function() {
                      var checked = jQuery(this).is(":checked");
                        jQuery(".onetime").each(function() {
                            checked ? jQuery(this).hide() : jQuery(this).show();
                        });
                        jQuery(".tokenization").each(function() {
                            checked ? jQuery(this).show() : jQuery(this).hide();
                        });
                    });
                  });
                </script>';

            return $html;
        }

        private function isPaymentMethodChange()
        {
            if ($this->get_option('subscriptions_enabled') !== 'yes') return false;
            if (empty($_GET['change_payment_method'])) return false;
            return true;
        }

        private function isSubscriptionPayment():bool
        {
            if (empty(WC()->cart->cart_contents) || $this->get_option('subscriptions_enabled') !== 'yes') return false;

            // Check if cart contains subscriptions
            foreach (WC()->cart->cart_contents as $cart_item) {
                $type = $cart_item['data']->get_type();
                if ($type !== 'subscription' && $type !== 'subscription_variation') {
                    continue;
                }
                return true;
            }

            return false;
        }

        private function getPayrexxInterface()
        {
            $this->register_autoloader();
            $platform = !empty($this->platform) ? $this->platform : \Payrexx\Communicator::API_URL_BASE_DOMAIN;
            return new \Payrexx\Payrexx($this->instance, $this->apiKey, '', $platform);
        }

        private function getCustomTransactionStatus($status) {
            return apply_filters('woo_payrexx_custom_transaction_status_' . $status, $status);
        }

        private function register_autoloader()
        {
            spl_autoload_register(function ($class) {
                $root = __DIR__ . '/payrexx-php-master';
                $classFile = $root . '/lib/' . str_replace('\\', '/', $class) . '.php';
                if (file_exists($classFile)) {
                    require_once $classFile;
                }
            });
        }

        private function deleteGatewayById($gatewayId):bool {
            $payrexx = $this->getPayrexxInterface();

            $gateway = new \Payrexx\Models\Request\Gateway();
            $gateway->setId($gatewayId);

            try {
                $payrexx->delete($gateway);
            } catch (\Payrexx\PayrexxException $e) {
                return false;
            }
            return true;
        }
    }
}

function wc_payrexx_add_to_gateways($gateways)
{
    $gateways[] = 'WC_Payrexx_Gateway';
    return $gateways;
}

add_filter('woocommerce_payment_gateways', 'wc_payrexx_add_to_gateways');
?>
